package com.sun.rowset;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.Reader;
import java.io.InputStream;
import java.sql.Timestamp;
import java.sql.Time;
import java.sql.Date;
import java.math.BigDecimal;
import java.util.Hashtable;
import java.sql.SQLException;
import javax.sql.rowset.Predicate;
import javax.sql.rowset.FilteredRowSet;
import java.io.Serializable;

public class FilteredRowSetImpl extends WebRowSetImpl implements Serializable, Cloneable, FilteredRowSet
{
    private Predicate p;
    private boolean onInsertRow;
    static final long serialVersionUID = 6178454588413509360L;
    
    public FilteredRowSetImpl() throws SQLException {
        this.onInsertRow = false;
    }
    
    public FilteredRowSetImpl(Hashtable hashtable) throws SQLException {
        super(hashtable);
        this.onInsertRow = false;
    }
    
    @Override
    public void setFilter(Predicate p) throws SQLException {
        this.p = p;
    }
    
    @Override
    public Predicate getFilter() {
        return this.p;
    }
    
    @Override
    protected boolean internalNext() throws SQLException {
        boolean internalNext = false;
        int i = this.getRow();
        while (i <= this.size()) {
            internalNext = super.internalNext();
            if (internalNext == 0 || this.p == null) {
                return internalNext;
            }
            if (this.p.evaluate(this)) {
                break;
            }
            ++i;
        }
        return internalNext;
    }
    
    @Override
    protected boolean internalPrevious() throws SQLException {
        boolean internalPrevious = false;
        int i = this.getRow();
        while (i > 0) {
            internalPrevious = super.internalPrevious();
            if (this.p == null) {
                return internalPrevious;
            }
            if (this.p.evaluate(this)) {
                break;
            }
            --i;
        }
        return internalPrevious;
    }
    
    @Override
    protected boolean internalFirst() throws SQLException {
        int n = super.internalFirst();
        if (this.p == null) {
            return n;
        }
        while (n && !this.p.evaluate(this)) {
            n = super.internalNext();
        }
        return n;
    }
    
    @Override
    protected boolean internalLast() throws SQLException {
        int n = super.internalLast();
        if (this.p == null) {
            return n;
        }
        while (n && !this.p.evaluate(this)) {
            n = super.internalPrevious();
        }
        return n;
    }
    
    @Override
    public boolean relative(int i) throws SQLException {
        boolean internalNext = false;
        boolean internalPrevious = false;
        if (this.getType() == 1003) {
            throw new SQLException(this.resBundle.handleGetObject("filteredrowsetimpl.relative").toString());
        }
        if (i > 0) {
            int j = 0;
            while (j < i) {
                if (this.isAfterLast()) {
                    return false;
                }
                internalNext = this.internalNext();
                ++j;
            }
        }
        else {
            while (i < 0) {
                if (this.isBeforeFirst()) {
                    return false;
                }
                internalPrevious = this.internalPrevious();
                ++i;
            }
            internalNext = internalPrevious;
        }
        if (i != 0) {
            this.notifyCursorMoved();
        }
        return internalNext;
    }
    
    @Override
    public boolean absolute(int n) throws SQLException {
        if (n == 0 || this.getType() == 1003) {
            throw new SQLException(this.resBundle.handleGetObject("filteredrowsetimpl.absolute").toString());
        }
        int n3;
        if (n > 0) {
            n3 = this.internalFirst();
            int i = 0;
            while (i < n - 1) {
                if (this.isAfterLast()) {
                    return false;
                }
                n3 = this.internalNext();
                ++i;
            }
        }
        else {
            int n2 = this.internalLast();
            while (n + 1 < 0) {
                if (this.isBeforeFirst()) {
                    return false;
                }
                n2 = this.internalPrevious();
                ++n;
            }
            n3 = n2;
        }
        this.notifyCursorMoved();
        return n3;
    }
    
    @Override
    public void moveToInsertRow() throws SQLException {
        this.onInsertRow = true;
        super.moveToInsertRow();
    }
    
    @Override
    public void updateInt(int n, int i) throws SQLException {
        if (this.onInsertRow && this.p != null && !this.p.evaluate(Integer.valueOf(i), n)) {
            throw new SQLException(this.resBundle.handleGetObject("filteredrowsetimpl.notallowed").toString());
        }
        super.updateInt(n, i);
        return;
    }
    
    @Override
    public void updateInt(String s, int n) throws SQLException {
        this.updateInt(this.findColumn(s), n);
    }
    
    @Override
    public void updateBoolean(int n, boolean b) throws SQLException {
        if (this.onInsertRow && this.p != null && !this.p.evaluate(Boolean.valueOf(b), n)) {
            throw new SQLException(this.resBundle.handleGetObject("filteredrowsetimpl.notallowed").toString());
        }
        super.updateBoolean(n, b);
        return;
    }
    
    @Override
    public void updateBoolean(String s, boolean b) throws SQLException {
        this.updateBoolean(this.findColumn(s), b);
    }
    
    @Override
    public void updateByte(int n, byte b) throws SQLException {
        if (this.onInsertRow && this.p != null && !this.p.evaluate(Byte.valueOf(b), n)) {
            throw new SQLException(this.resBundle.handleGetObject("filteredrowsetimpl.notallowed").toString());
        }
        super.updateByte(n, b);
        return;
    }
    
    @Override
    public void updateByte(String s, byte b) throws SQLException {
        this.updateByte(this.findColumn(s), b);
    }
    
    @Override
    public void updateShort(int n, short s) throws SQLException {
        if (this.onInsertRow && this.p != null && !this.p.evaluate(Short.valueOf(s), n)) {
            throw new SQLException(this.resBundle.handleGetObject("filteredrowsetimpl.notallowed").toString());
        }
        super.updateShort(n, s);
        return;
    }
    
    @Override
    public void updateShort(String s, short n) throws SQLException {
        this.updateShort(this.findColumn(s), n);
    }
    
    @Override
    public void updateLong(int n, long l) throws SQLException {
        if (this.onInsertRow && this.p != null && !this.p.evaluate(Long.valueOf(l), n)) {
            throw new SQLException(this.resBundle.handleGetObject("filteredrowsetimpl.notallowed").toString());
        }
        super.updateLong(n, l);
        return;
    }
    
    @Override
    public void updateLong(String s, long n) throws SQLException {
        this.updateLong(this.findColumn(s), n);
    }
    
    @Override
    public void updateFloat(int n, float f) throws SQLException {
        if (this.onInsertRow && this.p != null && !this.p.evaluate(Float.valueOf(f), n)) {
            throw new SQLException(this.resBundle.handleGetObject("filteredrowsetimpl.notallowed").toString());
        }
        super.updateFloat(n, f);
        return;
    }
    
    @Override
    public void updateFloat(String s, float n) throws SQLException {
        this.updateFloat(this.findColumn(s), n);
    }
    
    @Override
    public void updateDouble(int n, double d) throws SQLException {
        if (this.onInsertRow && this.p != null && !this.p.evaluate(Double.valueOf(d), n)) {
            throw new SQLException(this.resBundle.handleGetObject("filteredrowsetimpl.notallowed").toString());
        }
        super.updateDouble(n, d);
        return;
    }
    
    @Override
    public void updateDouble(String s, double n) throws SQLException {
        this.updateDouble(this.findColumn(s), n);
    }
    
    @Override
    public void updateBigDecimal(int n, BigDecimal bigDecimal) throws SQLException {
        if (this.onInsertRow && this.p != null && !this.p.evaluate(bigDecimal, n)) {
            throw new SQLException(this.resBundle.handleGetObject("filteredrowsetimpl.notallowed").toString());
        }
        super.updateBigDecimal(n, bigDecimal);
        return;
    }
    
    @Override
    public void updateBigDecimal(String s, BigDecimal bigDecimal) throws SQLException {
        this.updateBigDecimal(this.findColumn(s), bigDecimal);
    }
    
    @Override
    public void updateString(int n, String s) throws SQLException {
        if (this.onInsertRow && this.p != null && !this.p.evaluate(s, n)) {
            throw new SQLException(this.resBundle.handleGetObject("filteredrowsetimpl.notallowed").toString());
        }
        super.updateString(n, s);
        return;
    }
    
    @Override
    public void updateString(String s, String s2) throws SQLException {
        this.updateString(this.findColumn(s), s2);
    }
    
    @Override
    public void updateBytes(int n, byte[] array) throws SQLException {
        String concat = "";
        Byte[] array2 = new Byte[array.length];
        for (int i = 0; i < array.length; ++i) {
            array2[i] = Byte.valueOf(array[i]);
            concat = concat.concat(array2[i].toString());
        }
        if (this.onInsertRow && this.p != null && !this.p.evaluate(concat, n)) {
            throw new SQLException(this.resBundle.handleGetObject("filteredrowsetimpl.notallowed").toString());
        }
        super.updateBytes(n, array);
        return;
    }
    
    @Override
    public void updateBytes(String s, byte[] array) throws SQLException {
        this.updateBytes(this.findColumn(s), array);
    }
    
    @Override
    public void updateDate(int n, Date date) throws SQLException {
        if (this.onInsertRow && this.p != null && !this.p.evaluate(date, n)) {
            throw new SQLException(this.resBundle.handleGetObject("filteredrowsetimpl.notallowed").toString());
        }
        super.updateDate(n, date);
        return;
    }
    
    @Override
    public void updateDate(String s, Date date) throws SQLException {
        this.updateDate(this.findColumn(s), date);
    }
    
    @Override
    public void updateTime(int n, Time time) throws SQLException {
        if (this.onInsertRow && this.p != null && !this.p.evaluate(time, n)) {
            throw new SQLException(this.resBundle.handleGetObject("filteredrowsetimpl.notallowed").toString());
        }
        super.updateTime(n, time);
        return;
    }
    
    @Override
    public void updateTime(String s, Time time) throws SQLException {
        this.updateTime(this.findColumn(s), time);
    }
    
    @Override
    public void updateTimestamp(int n, Timestamp timestamp) throws SQLException {
        if (this.onInsertRow && this.p != null && !this.p.evaluate(timestamp, n)) {
            throw new SQLException(this.resBundle.handleGetObject("filteredrowsetimpl.notallowed").toString());
        }
        super.updateTimestamp(n, timestamp);
        return;
    }
    
    @Override
    public void updateTimestamp(String s, Timestamp timestamp) throws SQLException {
        this.updateTimestamp(this.findColumn(s), timestamp);
    }
    
    @Override
    public void updateAsciiStream(int n, InputStream inputStream, int n2) throws SQLException {
        if (this.onInsertRow && this.p != null && !this.p.evaluate(inputStream, n)) {
            throw new SQLException(this.resBundle.handleGetObject("filteredrowsetimpl.notallowed").toString());
        }
        super.updateAsciiStream(n, inputStream, n2);
        return;
    }
    
    @Override
    public void updateAsciiStream(String s, InputStream inputStream, int n) throws SQLException {
        this.updateAsciiStream(this.findColumn(s), inputStream, n);
    }
    
    @Override
    public void updateCharacterStream(int n, Reader reader, int n2) throws SQLException {
        if (this.onInsertRow && this.p != null && !this.p.evaluate(reader, n)) {
            throw new SQLException(this.resBundle.handleGetObject("filteredrowsetimpl.notallowed").toString());
        }
        super.updateCharacterStream(n, reader, n2);
        return;
    }
    
    @Override
    public void updateCharacterStream(String s, Reader reader, int n) throws SQLException {
        this.updateCharacterStream(this.findColumn(s), reader, n);
    }
    
    @Override
    public void updateBinaryStream(int n, InputStream inputStream, int n2) throws SQLException {
        if (this.onInsertRow && this.p != null && !this.p.evaluate(inputStream, n)) {
            throw new SQLException(this.resBundle.handleGetObject("filteredrowsetimpl.notallowed").toString());
        }
        super.updateBinaryStream(n, inputStream, n2);
        return;
    }
    
    @Override
    public void updateBinaryStream(String s, InputStream inputStream, int n) throws SQLException {
        this.updateBinaryStream(this.findColumn(s), inputStream, n);
    }
    
    @Override
    public void updateObject(int n, Object o) throws SQLException {
        if (this.onInsertRow && this.p != null && !this.p.evaluate(o, n)) {
            throw new SQLException(this.resBundle.handleGetObject("filteredrowsetimpl.notallowed").toString());
        }
        super.updateObject(n, o);
        return;
    }
    
    @Override
    public void updateObject(String s, Object o) throws SQLException {
        this.updateObject(this.findColumn(s), o);
    }
    
    @Override
    public void updateObject(int n, Object o, int n2) throws SQLException {
        if (this.onInsertRow && this.p != null && !this.p.evaluate(o, n)) {
            throw new SQLException(this.resBundle.handleGetObject("filteredrowsetimpl.notallowed").toString());
        }
        super.updateObject(n, o, n2);
        return;
    }
    
    @Override
    public void updateObject(String s, Object o, int n) throws SQLException {
        this.updateObject(this.findColumn(s), o, n);
    }
    
    @Override
    public void insertRow() throws SQLException {
        this.onInsertRow = false;
        super.insertRow();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        try {
            this.resBundle = JdbcRowSetResourceBundle.getJdbcRowSetResourceBundle();
        }
        catch (IOException cause) {
            throw new RuntimeException(cause);
        }
    }
}
